from pydantic import BaseModel, Field

from line_works.enums.yes_no_option import YesNoOption
from line_works.openapi.talk.configuration import Configuration

c = Configuration()


class LoginRequest(BaseModel):
    input_id: str = Field(alias="inputId")
    password: str
    keep_login: YesNoOption = Field(alias="keepLoginYn")
    remember_id: YesNoOption = Field(alias="rememberIdYn")
    access_url: str = Field(alias="accessUrl", default=c.host)

    class Config:
        use_enum_values = True
        populate_by_name = True



from line_works.config import UA

HOST = "wss://jp1-web-noti.worksmobile.com/wmqtt"
HEADERS = {
    "user-agent": UA,
    "origin": "https://talk.worksmobile.com",
    "sec-websocket-protocol": "mqtt",
}
KEEPALIVE_INTERVAL_SEC = 10



# coding: utf-8

"""
    LINE WORKS Talk API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from line_works.openapi.talk.models.email import Email
from line_works.openapi.talk.models.name import Name
from line_works.openapi.talk.models.organization import Organization
from line_works.openapi.talk.models.works_at import WorksAt
from typing import Optional, Set
from typing_extensions import Self

class MyInfo(BaseModel):
    """
    
    """ # noqa: E501
    tenant_id: StrictInt = Field(alias="tenantId")
    domain_id: StrictInt = Field(alias="domainId")
    contact_no: StrictInt = Field(alias="contactNo")
    read_only: StrictBool = Field(alias="readOnly")
    temp_id: StrictBool = Field(alias="tempId")
    name: Name
    i18n_name: StrictStr = Field(alias="i18nName")
    i18n_names: List[Dict[str, Any]] = Field(alias="i18nNames")
    photos: List[Dict[str, Any]]
    organizations: List[Organization]
    emails: List[Email]
    telephones: List[Dict[str, Any]]
    messengers: List[Dict[str, Any]]
    position: StrictStr
    department: StrictStr
    location: Optional[StrictStr]
    important: StrictBool
    executive: StrictBool
    photo_hash: StrictStr = Field(alias="photoHash")
    works_at: WorksAt = Field(alias="worksAt")
    access_limit: StrictBool = Field(alias="accessLimit")
    user_photo_modify: StrictBool = Field(alias="userPhotoModify")
    user_absence_modify: StrictBool = Field(alias="userAbsenceModify")
    organization: StrictStr
    groups: List[Dict[str, Any]]
    works_services: List[StrictStr] = Field(alias="worksServices")
    custom_fields: List[Dict[str, Any]] = Field(alias="customFields")
    profile_statuses: List[Dict[str, Any]] = Field(alias="profileStatuses")
    profile_statuses_v2: List[Dict[str, Any]] = Field(alias="profileStatusesV2")
    instance: StrictInt
    __properties: ClassVar[List[str]] = ["tenantId", "domainId", "contactNo", "readOnly", "tempId", "name", "i18nName", "i18nNames", "photos", "organizations", "emails", "telephones", "messengers", "position", "department", "location", "important", "executive", "photoHash", "worksAt", "accessLimit", "userPhotoModify", "userAbsenceModify", "organization", "groups", "worksServices", "customFields", "profileStatuses", "profileStatusesV2", "instance"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MyInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of name
        if self.name:
            _dict['name'] = self.name.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in organizations (list)
        _items = []
        if self.organizations:
            for _item_organizations in self.organizations:
                if _item_organizations:
                    _items.append(_item_organizations.to_dict())
            _dict['organizations'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in emails (list)
        _items = []
        if self.emails:
            for _item_emails in self.emails:
                if _item_emails:
                    _items.append(_item_emails.to_dict())
            _dict['emails'] = _items
        # override the default output from pydantic by calling `to_dict()` of works_at
        if self.works_at:
            _dict['worksAt'] = self.works_at.to_dict()
        # set to None if location (nullable) is None
        # and model_fields_set contains the field
        if self.location is None and "location" in self.model_fields_set:
            _dict['location'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MyInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "tenantId": obj.get("tenantId"),
            "domainId": obj.get("domainId"),
            "contactNo": obj.get("contactNo"),
            "readOnly": obj.get("readOnly"),
            "tempId": obj.get("tempId"),
            "name": Name.from_dict(obj["name"]) if obj.get("name") is not None else None,
            "i18nName": obj.get("i18nName"),
            "i18nNames": obj.get("i18nNames"),
            "photos": obj.get("photos"),
            "organizations": [Organization.from_dict(_item) for _item in obj["organizations"]] if obj.get("organizations") is not None else None,
            "emails": [Email.from_dict(_item) for _item in obj["emails"]] if obj.get("emails") is not None else None,
            "telephones": obj.get("telephones"),
            "messengers": obj.get("messengers"),
            "position": obj.get("position"),
            "department": obj.get("department"),
            "location": obj.get("location"),
            "important": obj.get("important"),
            "executive": obj.get("executive"),
            "photoHash": obj.get("photoHash"),
            "worksAt": WorksAt.from_dict(obj["worksAt"]) if obj.get("worksAt") is not None else None,
            "accessLimit": obj.get("accessLimit"),
            "userPhotoModify": obj.get("userPhotoModify"),
            "userAbsenceModify": obj.get("userAbsenceModify"),
            "organization": obj.get("organization"),
            "groups": obj.get("groups"),
            "worksServices": obj.get("worksServices"),
            "customFields": obj.get("customFields"),
            "profileStatuses": obj.get("profileStatuses"),
            "profileStatusesV2": obj.get("profileStatusesV2"),
            "instance": obj.get("instance")
        })
        return _obj





import asyncio
from ssl import create_default_context
from typing import Callable

import websockets
from pydantic import BaseModel, PrivateAttr
from websockets.asyncio.client import ClientConnection

from line_works.client import LineWorks
from line_works.logger import get_file_path_logger
from line_works.mqtt import config, packets
from line_works.mqtt.enums.packet_type import PacketType
from line_works.mqtt.exceptions import (
    LineWorksMQTTException,
    PacketParseException,
)
from line_works.mqtt.models.packet import MQTTPacket

logger = get_file_path_logger(__name__)


class MQTTClient(BaseModel):
    works: LineWorks
    _trace_func: dict[PacketType, Callable[[LineWorks, MQTTPacket], None]] = (
        PrivateAttr(default_factory=dict)
    )
    _ws: ClientConnection = PrivateAttr(default=None)
    _unique_ids: list[str] = PrivateAttr(default_factory=list)

    class Config:
        arbitrary_types_allowed = True

    def add_trace_func(
        self,
        packet_type: PacketType,
        f: Callable[[LineWorks, MQTTPacket], None],
    ) -> None:
        self._trace_func[packet_type] = f

    async def connect(self) -> None:
        self._ws = await websockets.connect(
            config.HOST,
            ssl=create_default_context(),
            additional_headers={
                "Cookie": self.works.cookie_str,
                **config.HEADERS,
            },
            subprotocols=["mqtt"],
            ping_interval=None,
        )

        await self._ws.send(packets.CONNECTION_PACKET)

        async with asyncio.TaskGroup() as tg:
            tg.create_task(self.__send_pingreq())
            tg.create_task(self.__listen())

    async def __send_pingreq(self) -> None:
        while True:
            await asyncio.sleep(config.KEEPALIVE_INTERVAL_SEC)
            await self._ws.send(packets.PINGREQ_PACKET)

    async def __listen(self) -> None:
        while True:
            message = await self._ws.recv()
            if isinstance(message, bytes):
                await self.__handle_binary_message(message)
            else:
                logger.debug(f"Received a non-binary message: {message}")

    async def __handle_binary_message(self, message: bytes) -> None:
        try:
            packet = MQTTPacket.parse_from_bytes(message)

            if packet.type == PacketType.PINGRESP:
                return

            if packet.type == PacketType.PUBLISH:
                try:
                    p = packet.payload
                    if p.unique_id in self._unique_ids:
                        return
                    elif p.unique_id:
                        self._unique_ids.append(p.unique_id)
                except PacketParseException as e:
                    logger.debug("packet parse error", exc_info=e)

            logger.debug(f"{packet=}")

            if f := self._trace_func.get(packet.type):
                f(self.works, packet)
        except LineWorksMQTTException as e:
            logger.error(
                "Error while handling binary message. "
                "Failed to parse or process the MQTT packet.",
                exc_info=e,
            )
        except Exception as e:
            logger.error("error", exc_info=e)



import json
from typing import Optional

from pydantic import Field

from line_works.exceptions import LogicException
from line_works.mqtt.enums.notification_type import NotificationType
from line_works.mqtt.models.payload.badge import BadgePayload
from line_works.openapi.talk.models.sticker import Sticker


class MessagePayload(BadgePayload):
    bot_info: str = Field(alias="botInfo", default="")
    channel_no: Optional[int] = Field(alias="chNo", default=None)
    channel_photo_path: str = Field(alias="chPhotoPath", default="")
    channel_title: str = Field(alias="chTitle", default="")
    channel_type: Optional[int] = Field(alias="chType", default=None)
    create_time: Optional[int] = Field(alias="createTime", default="")
    extras: str = Field(default="")
    from_photo_hash: str = Field(alias="fromPhotoHash", default="")
    from_user_no: Optional[int] = Field(alias="fromUserNo", default=None)
    message_no: Optional[int] = Field(alias="messageNo", default=None)
    notification_id: str = Field(alias="notification-id", default="")

    class Config:
        populate_by_name = True

    @property
    def unique_id(self) -> str:
        return f"{self.loc_key}_{self.notification_id}"

    @property
    def extras_dict(self) -> dict:
        return json.loads(self.extras) if self.extras else {}

    @property
    def sticker(self) -> Sticker:
        if self.notification_type == NotificationType.NOTIFICATION_STICKER:
            return Sticker(**self.extras_dict)
        raise LogicException(
            f"Invalid notification type: {self.notification_type}. "
            f"Expected {NotificationType.NOTIFICATION_STICKER}."
        )



import json
import struct
from typing import Any, Optional, Self

from pydantic import BaseModel, Field

from line_works.mqtt.enums.packet_type import PacketType
from line_works.mqtt.exceptions import PacketParseException
from line_works.mqtt.models.payload import (
    NOTIFICATION_TYPE_MODEL_MAPPING,
    PayloadTypes,
)


class MQTTPacket(BaseModel):
    type: PacketType
    flags: int
    remaining_length: int
    raw_payload: Optional[bytes]
    raw_packet: bytes = Field(repr=False)

    def validate_publish_payload(self) -> None:
        if self.type != PacketType.PUBLISH:
            raise PacketParseException(
                f"Expected packet type {PacketType.PUBLISH}, "
                f"but got {self.type}."
            )

    @property
    def topic_length(self) -> int:
        self.validate_publish_payload()
        if not self.raw_payload:
            raise PacketParseException("Payload is missing or invalid.")

        topic_length: int = struct.unpack("!H", self.raw_payload[0:2])[0]
        return topic_length

    @property
    def topic_name(self) -> str:
        self.validate_publish_payload()
        if not self.raw_payload:
            raise PacketParseException("Payload is missing or invalid.")

        topic: str = self.raw_payload[2 : 2 + self.topic_length].decode(
            "utf-8"
        )
        return topic

    @property
    def publish_payload(self) -> dict[str, Any]:
        self.validate_publish_payload()
        if not self.raw_payload:
            raise PacketParseException("Payload is missing or invalid.")

        pos = 2 + self.topic_length

        qos = (self.flags & 0x06) >> 1
        if qos > 0:
            if len(self.raw_payload) < pos + 2:
                raise PacketParseException(
                    "Packet too short for QoS > 0: "
                    f"expected at least {pos + 2} bytes, "
                    f"but got {len(self.raw_payload)} bytes. "
                    f"raw_payload: {self.raw_payload.hex()}"
                )
            pos += 2

        payload = self.raw_payload[pos:].decode("utf-8")
        if payload:
            j: dict[str, Any] = json.loads(payload)
            return j

        return {}

    @property
    def payload(self) -> PayloadTypes:
        p = self.publish_payload
        if not (n_type := p.get("nType")):
            raise PacketParseException(f"invalid payload: {p}")

        if not (p_model := NOTIFICATION_TYPE_MODEL_MAPPING.get(n_type)):
            raise PacketParseException(f"invalid notification type: {n_type}")

        return p_model.model_validate(self.publish_payload)  # type: ignore

    @classmethod
    def parse_from_bytes(cls, data: bytes) -> Self:
        if (data_length := len(data)) < 2:
            raise PacketParseException(
                f"Data size is too small: {data_length} bytes"
            )

        packet_type = (data[0] & 0xF0) >> 4
        flags = data[0] & 0x0F

        remaining_length = 0
        multiplier = 1
        pos = 1

        while pos < len(data):
            byte = data[pos]
            remaining_length += (byte & 0x7F) * multiplier
            multiplier *= 128
            pos += 1

            if byte & 0x80 == 0:
                break

        raw_payload = (
            data[pos : pos + remaining_length]
            if remaining_length > 0
            else None
        )

        return cls(
            type=PacketType(packet_type),
            flags=flags,
            remaining_length=remaining_length,
            raw_payload=raw_payload,
            raw_packet=data,
        )



from typing import Self

from line_works.enums.message_type import MessageType
from line_works.openapi.talk.models.caller import Caller
from line_works.openapi.talk.models.send_message_request import (
    SendMessageRequest as BaseSendMessageRequest,
)
from line_works.openapi.talk.models.sticker import Sticker


class SendMessageRequest(BaseSendMessageRequest):
    class Config:
        use_enum_values = True

    @classmethod
    def text_message(cls, caller: Caller, channel_no: int, text: str) -> Self:
        return cls(
            channel_no=channel_no,
            content=text,
            caller=caller,
            type=MessageType.TEXT,
        )

    @classmethod
    def sticker_message(
        cls, caller: Caller, channel_no: int, sticker: Sticker
    ) -> Self:
        return cls(
            channel_no=channel_no,
            caller=caller,
            extras=sticker.model_dump_json(by_alias=True),
            type=MessageType.STICKER,
        )



# coding: utf-8

# flake8: noqa

"""
    LINE WORKS Talk API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from line_works.openapi.talk.api.default_api import DefaultApi

# import ApiClient
from line_works.openapi.talk.api_response import ApiResponse
from line_works.openapi.talk.api_client import ApiClient
from line_works.openapi.talk.configuration import Configuration
from line_works.openapi.talk.exceptions import OpenApiException
from line_works.openapi.talk.exceptions import ApiTypeError
from line_works.openapi.talk.exceptions import ApiValueError
from line_works.openapi.talk.exceptions import ApiKeyError
from line_works.openapi.talk.exceptions import ApiAttributeError
from line_works.openapi.talk.exceptions import ApiException

# import models into sdk package
from line_works.openapi.talk.models.base_response import BaseResponse
from line_works.openapi.talk.models.caller import Caller
from line_works.openapi.talk.models.email import Email
from line_works.openapi.talk.models.get_channel_members_request import GetChannelMembersRequest
from line_works.openapi.talk.models.get_channel_members_response import GetChannelMembersResponse
from line_works.openapi.talk.models.leave_channel_request import LeaveChannelRequest
from line_works.openapi.talk.models.leave_channel_response import LeaveChannelResponse
from line_works.openapi.talk.models.member import Member
from line_works.openapi.talk.models.my_info import MyInfo
from line_works.openapi.talk.models.name import Name
from line_works.openapi.talk.models.organization import Organization
from line_works.openapi.talk.models.send_message_request import SendMessageRequest
from line_works.openapi.talk.models.send_message_response import SendMessageResponse
from line_works.openapi.talk.models.sent_message import SentMessage
from line_works.openapi.talk.models.sticker import Sticker
from line_works.openapi.talk.models.user import User
from line_works.openapi.talk.models.works_at import WorksAt



CONNECTION_PACKET = bytearray(
    [
        0x10,  # CONNECT パケットタイプ
        0x0C,  # 残りのパケット長
        0x00,
        0x04,  # プロトコル名長
        0x4D,
        0x51,
        0x54,
        0x54,  # "MQTT"
        0x04,  # プロトコルレベル (MQTTv3.1.1)
        0x02,  # 接続フラグ
        0x00,
        0x3C,  # キープアライブ (60秒)
        0x00,
        0x00,  # クライアントID長 (空のクライアントID)
    ]
)

PINGREQ_PACKET = bytearray([0xC0, 0x00])



SESSION_DIR = ".sessions"

UA = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36"
HEADERS = {
    "Accept": "*/*",
    "Accept-Language": "ja;q=0.8",
    "User-Agent": UA,
}



from abc import ABC, abstractmethod

from pydantic import BaseModel, Field

from line_works.mqtt.enums.notification_type import NotificationType


class BasePayload(BaseModel, ABC):
    domain_id: int = Field(alias="domain_id")
    loc_args0: str = Field(alias="loc-args0", default="")
    loc_args1: str = Field(alias="loc-args1", default="")
    loc_key: str = Field(alias="loc-key", default="")
    s_type: int = Field(alias="sType")
    ocn: int
    notification_type: NotificationType = Field(alias="nType")

    class Config:
        populate_by_name = True

    @property
    @abstractmethod
    def unique_id(self) -> str:
        pass



# coding: utf-8

"""
    LINE WORKS Talk API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class Member(BaseModel):
    """
    Member
    """ # noqa: E501
    user_no: StrictInt = Field(alias="userNo")
    name: StrictStr
    i18n_name: Optional[StrictStr] = Field(alias="i18nName")
    i18n_names: Optional[List[Any]] = Field(alias="i18nNames")
    nick_name: Optional[StrictStr] = Field(alias="nickName")
    position: Optional[StrictStr]
    i18n_positions: Optional[List[Any]] = Field(alias="i18nPositions")
    group_name: Optional[StrictStr] = Field(alias="groupName")
    i18n_group_names: Optional[List[Any]] = Field(alias="i18nGroupNames")
    group_position: Optional[StrictStr] = Field(alias="groupPosition")
    i18n_group_positions: Optional[List[Any]] = Field(alias="i18nGroupPositions")
    photo_hash: Optional[StrictStr] = Field(alias="photoHash")
    join: StrictBool
    join_time: StrictInt = Field(alias="joinTime")
    update_time: StrictInt = Field(alias="updateTime")
    domain_id: Optional[StrictInt] = Field(alias="domainId")
    property_flag: StrictInt = Field(alias="propertyFlag")
    domain_name: Optional[StrictStr] = Field(alias="domainName")
    i18n_domain_names: Optional[List[Any]] = Field(alias="i18nDomainNames")
    service_type: StrictStr = Field(alias="serviceType")
    relation_status: StrictStr = Field(alias="relationStatus")
    tenant_id: StrictInt = Field(alias="tenantId")
    guest: StrictBool
    __properties: ClassVar[List[str]] = ["userNo", "name", "i18nName", "i18nNames", "nickName", "position", "i18nPositions", "groupName", "i18nGroupNames", "groupPosition", "i18nGroupPositions", "photoHash", "join", "joinTime", "updateTime", "domainId", "propertyFlag", "domainName", "i18nDomainNames", "serviceType", "relationStatus", "tenantId", "guest"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Member from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if i18n_name (nullable) is None
        # and model_fields_set contains the field
        if self.i18n_name is None and "i18n_name" in self.model_fields_set:
            _dict['i18nName'] = None

        # set to None if i18n_names (nullable) is None
        # and model_fields_set contains the field
        if self.i18n_names is None and "i18n_names" in self.model_fields_set:
            _dict['i18nNames'] = None

        # set to None if nick_name (nullable) is None
        # and model_fields_set contains the field
        if self.nick_name is None and "nick_name" in self.model_fields_set:
            _dict['nickName'] = None

        # set to None if position (nullable) is None
        # and model_fields_set contains the field
        if self.position is None and "position" in self.model_fields_set:
            _dict['position'] = None

        # set to None if i18n_positions (nullable) is None
        # and model_fields_set contains the field
        if self.i18n_positions is None and "i18n_positions" in self.model_fields_set:
            _dict['i18nPositions'] = None

        # set to None if group_name (nullable) is None
        # and model_fields_set contains the field
        if self.group_name is None and "group_name" in self.model_fields_set:
            _dict['groupName'] = None

        # set to None if i18n_group_names (nullable) is None
        # and model_fields_set contains the field
        if self.i18n_group_names is None and "i18n_group_names" in self.model_fields_set:
            _dict['i18nGroupNames'] = None

        # set to None if group_position (nullable) is None
        # and model_fields_set contains the field
        if self.group_position is None and "group_position" in self.model_fields_set:
            _dict['groupPosition'] = None

        # set to None if i18n_group_positions (nullable) is None
        # and model_fields_set contains the field
        if self.i18n_group_positions is None and "i18n_group_positions" in self.model_fields_set:
            _dict['i18nGroupPositions'] = None

        # set to None if photo_hash (nullable) is None
        # and model_fields_set contains the field
        if self.photo_hash is None and "photo_hash" in self.model_fields_set:
            _dict['photoHash'] = None

        # set to None if domain_id (nullable) is None
        # and model_fields_set contains the field
        if self.domain_id is None and "domain_id" in self.model_fields_set:
            _dict['domainId'] = None

        # set to None if domain_name (nullable) is None
        # and model_fields_set contains the field
        if self.domain_name is None and "domain_name" in self.model_fields_set:
            _dict['domainName'] = None

        # set to None if i18n_domain_names (nullable) is None
        # and model_fields_set contains the field
        if self.i18n_domain_names is None and "i18n_domain_names" in self.model_fields_set:
            _dict['i18nDomainNames'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Member from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "userNo": obj.get("userNo"),
            "name": obj.get("name"),
            "i18nName": obj.get("i18nName"),
            "i18nNames": obj.get("i18nNames"),
            "nickName": obj.get("nickName"),
            "position": obj.get("position"),
            "i18nPositions": obj.get("i18nPositions"),
            "groupName": obj.get("groupName"),
            "i18nGroupNames": obj.get("i18nGroupNames"),
            "groupPosition": obj.get("groupPosition"),
            "i18nGroupPositions": obj.get("i18nGroupPositions"),
            "photoHash": obj.get("photoHash"),
            "join": obj.get("join"),
            "joinTime": obj.get("joinTime"),
            "updateTime": obj.get("updateTime"),
            "domainId": obj.get("domainId"),
            "propertyFlag": obj.get("propertyFlag"),
            "domainName": obj.get("domainName"),
            "i18nDomainNames": obj.get("i18nDomainNames"),
            "serviceType": obj.get("serviceType"),
            "relationStatus": obj.get("relationStatus"),
            "tenantId": obj.get("tenantId"),
            "guest": obj.get("guest")
        })
        return _obj





from pydantic import Field

from line_works.mqtt.models.payload._base import BasePayload


class ServicePayload(BasePayload):
    detail_message_exist: int = Field(alias="detailMessageExist")
    icon_url: str = Field(alias="iconUrl")
    mqtt_target_device_type_list0: str = Field(
        alias="mqttTargetDeviceTypeList0"
    )
    mqtt_target_device_type_list1: str = Field(
        alias="mqttTargetDeviceTypeList1"
    )
    mqtt_target_device_type_list2: str = Field(
        alias="mqttTargetDeviceTypeList2"
    )
    notification_no: int = Field(alias="notificationNo")
    service_type: int = Field(alias="serviceType")
    show_red_dot: int = Field(alias="showRedDot")

    class Config:
        populate_by_name = True

    @property
    def unique_id(self) -> str:
        return f"{self.loc_key}_{self.notification_no}"



from typing import Type

from line_works.mqtt.enums.notification_type import NotificationType
from line_works.mqtt.models.payload.badge import BadgePayload
from line_works.mqtt.models.payload.message import (
    MessagePayload,
)
from line_works.mqtt.models.payload.service import ServicePayload

PayloadTypes = MessagePayload | ServicePayload | BadgePayload

NOTIFICATION_TYPE_MODEL_MAPPING: dict[int, Type[PayloadTypes]] = {
    NotificationType.NOTIFICATION_MESSAGE.value: MessagePayload,
    NotificationType.NOTIFICATION_STICKER.value: MessagePayload,
    NotificationType.NOTIFICATION_IMAGE.value: MessagePayload,
    NotificationType.NOTIFICATION_EMOJI.value: MessagePayload,
    NotificationType.NOTIFICATION_BADGE.value: BadgePayload,
    NotificationType.NOTIFICATION_SERVICE.value: ServicePayload,
}



from enum import IntEnum


class NotificationType(IntEnum):
    NORMAL = 101
    AWAY = 102  # TODO: 不在メッセージの詳細仕様を確認
    LEAVE = 202
    INVITE = 203
    KICK = 204

    # コマンドメッセージタイプ
    CMD_READ = 93004

    # 通知メッセージタイプ
    NOTIFICATION_MESSAGE = 1
    NOTIFICATION_STICKER = 18
    NOTIFICATION_FILE = 16
    NOTIFICATION_SERVICE = 100
    NOTIFICATION_EMOJI = 27
    NOTIFICATION_IMAGE = 11
    NOTIFICATION_BADGE = 41  # TODO: バッジ更新通知の詳細仕様を確認



from pydantic import Field

from line_works.mqtt.models.payload._base import BasePayload


class BadgePayload(BasePayload):
    a_badge: int = Field(alias="aBadge")
    badge: int
    c_badge: int = Field(alias="cBadge")
    h_badge: int = Field(alias="hBadge")
    m_badge: int = Field(alias="mBadge")
    token: str
    user_no: int = Field(alias="userNo")
    wpa_badge: int = Field(alias="wpaBadge")

    class Config:
        populate_by_name = True

    @property
    def unique_id(self) -> str:
        return f"{self.loc_key}_{self.token}"



import json
from os import makedirs
from typing import TYPE_CHECKING, Callable, ParamSpec, TypeVar

from line_works import config

if TYPE_CHECKING:
    from line_works import LineWorks

T = TypeVar("T")
P = ParamSpec("P")


def save_cookie(func: Callable[P, T]) -> Callable[P, T]:
    """save session with json file

    Args:
        func (Callable): function

    Returns:
        Callable: wrapper function
    """

    def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
        result = func(*args, **kwargs)

        m: LineWorks = args[0]  # type: ignore

        makedirs(config.SESSION_DIR, exist_ok=True)
        with open(m.cookie_path, "w") as json_file:
            json.dump(m.session.cookies.get_dict(), json_file, indent=4)  # type: ignore

        return result

    return wrapper



from enum import IntEnum


class PacketType(IntEnum):
    CONNECT = 1
    CONNACK = 2
    PUBLISH = 3
    PUBACK = 4
    UNKNWON = 6  # TODO: Rename when you know
    UNKNWON_2 = 7  # TODO: Rename when you know
    SUBSCRIBE = 8
    SUBACK = 9
    PINGREQ = 12
    PINGRESP = 13
    DISCONNECT = 14



import json
from os import makedirs
from os.path import exists
from os.path import join as path_join
from typing import Any

from pydantic import BaseModel, Field, PrivateAttr
from requests import HTTPError, Session

from line_works import config
from line_works.decorator import save_cookie
from line_works.enums.yes_no_option import YesNoOption
from line_works.exceptions import LoginException
from line_works.logger import get_file_path_logger
from line_works.openapi.talk.api.default_api import DefaultApi as TalkApi
from line_works.openapi.talk.api_client import ApiClient as TalkApiClient
from line_works.openapi.talk.models.caller import Caller
from line_works.openapi.talk.models.send_message_response import (
    SendMessageResponse,
)
from line_works.openapi.talk.models.sticker import Sticker
from line_works.requests.login import LoginRequest
from line_works.requests.send_message import SendMessageRequest
from line_works.urls.auth import AuthURL

logger = get_file_path_logger(__name__)


class LineWorks(BaseModel, TalkApi):
    works_id: str
    password: str = Field(repr=False)
    keep_login: YesNoOption = Field(repr=False, default=YesNoOption.YES)
    remember_id: YesNoOption = Field(repr=False, default=YesNoOption.YES)
    tenant_id: int = Field(init=False, default=0)
    domain_id: int = Field(init=False, default=0)
    contact_no: int = Field(init=False, default=0)
    session: Session = Field(init=False, repr=False, default_factory=Session)
    api_client: TalkApiClient = Field(
        init=False, repr=False, default_factory=TalkApiClient
    )
    _caller: Caller = PrivateAttr()

    class Config:
        arbitrary_types_allowed = True

    @property
    def session_dir(self) -> str:
        return path_join(config.SESSION_DIR, self.works_id)

    @property
    def cookie_path(self) -> str:
        return path_join(self.session_dir, "cookie.json")

    @property
    def cookie_str(self) -> str:
        return "; ".join(f"{k}={v}" for k, v in self.session.cookies.items())

    def model_post_init(self, __context: Any) -> None:
        makedirs(self.session_dir, exist_ok=True)
        self.session.headers.update(config.HEADERS)

        if exists(self.cookie_path):
            # login with cookie
            with open(self.cookie_path) as j:
                c = json.load(j)
            self.session.cookies.update(c)

        try:
            my_info = self.get_my_info()
        except Exception:
            self.session.cookies.clear()
            self.login_with_id()

        TalkApi.__init__(self)
        for k, v in config.HEADERS.items():
            self.api_client.set_default_header(k, v)
        self.api_client.set_default_header("Cookie", self.cookie_str)

        my_info = self.get_my_info()
        self.tenant_id = my_info.tenant_id
        self.domain_id = my_info.domain_id
        self.contact_no = my_info.contact_no
        self._caller = Caller(
            domain_id=self.domain_id, user_no=self.contact_no
        )

        logger.info(f"login success: {self!r}")

    @save_cookie
    def login_with_id(self) -> None:
        self.session.get(AuthURL.LOGIN)

        try:
            r = self.session.post(
                AuthURL.LOGIN_PROCESS_V2,
                data=LoginRequest(
                    input_id=self.works_id,
                    password=self.password,
                    keep_login=self.keep_login,
                    remember_id=self.remember_id,
                ).model_dump(by_alias=True),
            )
            r.raise_for_status()
        except HTTPError as e:
            raise LoginException(e)

    def send_text_message(self, to: int, text: str) -> SendMessageResponse:
        return self.send_message(
            send_message_request=SendMessageRequest.text_message(
                self._caller, to, text
            )
        )

    def send_sticker_message(
        self, to: int, sticker: Sticker
    ) -> SendMessageResponse:
        return self.send_message(
            send_message_request=SendMessageRequest.sticker_message(
                self._caller, to, sticker
            )
        )
